/*
** iksemel (XML Parser for Jabber)
** Copyright (C) 2000-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of GNU Lesser General Public License.
**
** self tests
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#include "iksemel.h"

int total_bugs = 0;

void test_sha(void);
void test_str(void);
void test_iks(void);
void test_parser(void);


int main(int argc, char *argv[])
{
	setlocale(LC_ALL, "");

	test_sha();
	test_str();
	test_iks();
	test_parser();


	if(total_bugs)
	{
		printf("Encountered %d bugs :-(\n", total_bugs);
		return 1;
	}
	return 0;
}


void test_sha(void)
{
	struct lala
	{
		char *str, *hash;
	} known_hashes[] =
	{
		{ "abc", "A9993E364706816ABA3E25717850C26C9CD0D89D" },
		{ "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		  "84983E441C3BD26EBAAE4AA1F95129E5E54670F1" },
		{ NULL, NULL }
	};

	int bug = 0, i = 0;
	char buf[42];

	printf("Testing SHA1 implementation... ");
	fflush(stdout);

	while(known_hashes[i].str)
	{
		iks_sha(known_hashes[i].str, buf);
		if(strcmp(buf, known_hashes[i].hash) != 0)
		{
			if(bug == 0)
			{
				total_bugs++;
				printf("failed.\n");
			}
			bug++;
			printf("iks_sha(\"%s\", buf);\n", known_hashes[i].str);
			printf(" Result:   %s\n", buf);
			printf(" Expected: %s\n", known_hashes[i].hash);
		}
		i++;
	}

	if(bug == 0) printf("OK.\n");
}


void test_str(void)
{
	ikstr *s;
	char *t;

	printf("Testing string functions... ");
	fflush(stdout);

	s = iks_str_new(NULL);
	iks_str_add(s, "");
	iks_str_add(s, "<message to='");
	iks_str_add(s, NULL);
	iks_spool(s, NULL, "ydobon@jabber.org", "'>", "", "<thread>lala</thread><body>", s);
	iks_str_add(s, "hello!");
	iks_spool(s, s);
	iks_str_add(s, "</body></message>");
	t = iks_str_print(s);

	if(!s || !t || strcmp(t, "<message to='ydobon@jabber.org'><thread>lala</thread><body>hello!</body></message>") != 0)
	{
		total_bugs++;
		printf("failed.\n");
	}
	else
	{
		printf("OK.\n");
	}

	iks_str_delete(s);
}


void test_iks(void)
{
	static char xml[] =
		"<iq type='result' to='ydobon@jabber.org'><query xmlns='jabber:iq:version'>"
		"<name>TestClient</name><os>SuxOS 2000</os><version><stable solidity='rock'/>"
		"1.2.0 patchlevel 2</version></query></iq>";

	iks *x, *y, *z;
	ikstr *s;
	char *t;

	printf("Testing XML node functions... ");
	fflush(stdout);

	x = iks_new("iq");
	iks_insert_attrib(x, "type", "result");
	iks_insert_attrib(x, "to", "ydobon@jabber.org");
	y = iks_new_pool(iks_pool(x), "query");
	iks_insert_cdata(iks_insert(y, "name"), "TestClient", 10);
	iks_insert_cdata(iks_insert(y, "os"), "SuxOS", -1);
	z = iks_insert(y, "version");
	iks_insert(z, "stable");
	iks_insert_cdata(z, "1.2", 3);
	iks_insert_cdata(z, ".0 patchlevel 2", -1);
	iks_insert_node(x, y);
	z = iks_find(y, "os");
	iks_insert_cdata(z, " 2000", 5);
	z = iks_next(z);
	z = iks_find(z, "stable");
	iks_insert_attrib(z, "solidity", "rock");
	z = iks_parent(iks_parent(z));
	iks_insert_attrib(z, "xmlns", "jabber:iq:version");

	s = iks_string(iks_pool(x), x);
	t = iks_str_print(s);

	if(!t || strcmp(t, xml) != 0)
	{
		total_bugs++;
		printf("failed.\n");
		printf("Result:   %s\n", t);
		printf("Expected: %s\n", xml);
	}
	else
	{
		printf("OK.\n");
	}

	iks_delete(x);
}


void test_parser(void)
{
	static char buf[] = "<presence id='JCOM_11' to='lala@j.org' type='available'><status>&quot; &lt;online&amp;dangerous&gt; &quot;</status>meow<a><b c='d'/></a><test/></presence>";
	char *buf2;
	iksparser *prs;
	iks *x;
	int i = 0;

	printf("Testing XML parser... ");
	fflush(stdout);

	buf2 = strdup(buf);
	prs = iks_dom_new(&x);
	while(buf2[i] != '\0')
	{
		iks_parse(prs, buf2 + i, 1, 0);
		i++;
	}
	iks_parse(prs, NULL, 0, 1);

	free(buf2);
	buf2 = iks_str_print(iks_string(iks_pool(x), x));

	if(!buf2 || strcmp(buf2, buf) != 0)
	{
		total_bugs++;
		printf("failed.\n");
		printf("Result:   %s\n", buf2);
		printf("Expected: %s\n", buf);
	}
	else
	{
		printf("OK.\n");
	}

	iks_delete(x);
	iks_parser_delete(prs);
}
