/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
*
** common structures, macros, and prototypes
*/

#ifndef JABBERX_COMMON_H
#define JABBERX_COMMON_H

#undef NONEWUI

#include <config.h>

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <iksemel.h>


#ifdef HAVE_SG
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <string.h>
#include <ctype.h>

#include "libks.h"
#include "registerSSL.h"
#include "debug.h"

#endif





#ifndef bool
typedef int bool_t;
#define bool bool_t
#endif

/*
** Structures
*/

typedef struct listitem_s
{
	struct listitem_s *next;
	struct listitem_s *prev;
} listitem;

typedef struct
{
	listitem *first;
	listitem *last;
	int count;
} list;

enum gattrs
{
	GA_BODY = 0,
	GA_HEADER,
	GA_SYSMSG,
	GA_MENU,
	GA_NOTICE
};

#ifdef HAVE_SG

#define MAX_MSGSIZE     4096
#define MAX_NAME        1024

#define def_certFile    "cli-cert.pem"
#define def_keyFile     "cli-key.pem"
#define def_caFile      "cacert.pem"
#define def_caPath      "./demoCA"
#define def_ctxFile     "ctx.2"

typedef struct KEYSTONE
{
  SSLInfo sslInfo;
  AddrRec regAddr;
  KSCtx *ksCtx;
  char *grpName[MAX_GROUPS];
  int request[MAX_GROUPS];
  int numGrps;
} keystoneSG;

keystoneSG keystoneclient;
#endif

/*
GA_SYSMSG
GA_NORMAL
GA_JID
GA_MESSAGE
GA_NOTICE
GA_HELP
*/

enum textbuftypes
{
	TB_MAIN = 0,
	TB_GCHAT,
	TB_CHAT
};

typedef struct
{
	enum gattrs startattr;
	int startofs;
	int visiblechars;
} lineinfo;

typedef struct
{
	listitem node;
	char *buf;
	iksid *id;
        char *thread;
	enum textbuftypes type;
	int lines, curline;
	int bytes;
	int bufmax;
	int curpart;
	enum gattrs curattr;
	lineinfo *lineinfos;
	int limax;
	bool insync;
} textbuf;


enum netstates
{
	NET_OFF,
	NET_CONNECT,
	NET_STREAM,
	NET_REG,
	NET_AUTH,
	NET_ON
};

struct netdata
{
	iksparser *parser;
	iksid *id;
	enum netstates state;
	int regflag;



};

enum jusersubs
{
	JUSER_NONE,
	JUSER_TO,
	JUSER_FROM,
	JUSER_BOTH
};

typedef struct jgroup_struct
{
	listitem node;
	ikspool *p;
	char *name;
} jgroup;

typedef struct jgroupref_struct
{
	listitem node;
	ikspool *p;
	jgroup *ref;
} jgroupref;

typedef struct juser_struct
{
	listitem node;
	ikspool *p;
	iksid *id;
	char *name;
	enum jusersubs subs;
	enum ikshowtype show;
	char *status;
	list *groups;
} juser;


struct prfdata
{
	char *user;
	char *pass;
	char *logfile;
	char *resource;
};


struct optionsdata
{
	int show_sent;
	int hide_resource;
	int compact_lines;
	int no_colors;
	int resend_presence;
	int timestamps;
	int auto_chat;
	int popup_chat;
	char *codeset;
	char *on_message;
	char *on_roster;
	int fetch_options;
};




/*
** Globals
*/

extern struct optionsdata options;
extern struct prfdata prf;
extern struct netdata net;

extern int ui_w, ui_h;

extern list *textbufs;
extern textbuf *mainbuf;
extern textbuf *curbuf;




/*
** Prototypes
*/

int prf_setup(int argc, char *argv[]);


void cmd_quit(char *arg);
int ui_run(void);
void gui_prompt(void);
void gui_refresh(void);
void ui_status(void);
void ui_set_tab_content(char *fmt, ...);


/* Prototypes of txtbuf.c for all: */

textbuf *tb_new(void);
#define ui_buffer_new tb_new
void tb_show(textbuf *tb);
#define ui_buffer_show tb_show
void tb_delete(textbuf *tb);
#define ui_buffer_delete tb_delete

/* Prototypes of txtbuf.c for ui.c: */

void tb_set_attr(textbuf *tb, enum gattrs attr);
void tb_getname(char *dest, size_t len, textbuf *tb);
void tb_puts(textbuf *tb, char *str);
#define ui_buffer_puts tb_puts
void tb_render(textbuf *tb);
void tb_forward(textbuf *tb, int lines);
#define ui_buffer_forward tb_forward
void tb_backward(textbuf *tb, int lines);
#define ui_buffer_backward tb_backward
void tb_bottom(textbuf *tb);
#define ui_buffer_bottom tb_bottom
void tb_top(textbuf *tb);
#define ui_buffer_top tb_top

/* Prototypes of ui.c for all: */

#define UIF_NORMAL     0 
#define UIF_STRIPNL    1   // Strip newline at the end 
#define UIF_NEWLINE    2   // Add a newline after it
#define UIF_EXTRANL    4   // Add a second newline after it, if !options.compact_lines
#define UIF_COPYMAIN   8   // Also output to to mainbuf/curbuf, if different
#define UIF_SETATTR   16   // Set color in attr
#define UIF_TIMESTAMP 32   // Add a timestamp at the beginning if option.timestamps is set.

void ui_xprint(int uiflags, textbuf *tb, enum gattrs attr, char *fmt, ...);

#define gui_print(fmt,a...) ui_xprint(UIF_NORMAL, mainbuf, 0, fmt, ##a)
#define gui_ts_print(fmt,a...) ui_xprint(UIF_TIMESTAMP, mainbuf, 0, fmt, ##a)
#define gui_println(fmt,a...) ui_xprint(UIF_STRIPNL|UIF_NEWLINE, mainbuf, 0, fmt, ##a)
#define gui_attr(attr) ui_xprint(UIF_SETATTR, mainbuf, attr, NULL)
#define ui_print(tb,fmt,a...) ui_xprint(UIF_NORMAL, tb, 0, fmt, ##a)
#define gui_xprint(uiflags,attr,fmt,a...) ui_xprint(uiflags, mainbuf, attr, fmt,##a) 

/* Prototypes from ui.c for txtbuf.c: */
void ui_win_reset();
void ui_win_next(int scroll);
void ui_win_refresh();
void ui_win_render(enum gattrs attr, char *text, int max, int shift, int ofs)  ;

/* Prototype from ui.c for prefs.c:  */
void optakt_codeset(char *name,void *data,int gui);

void cmd_connect(char *arg);
#ifdef HAVE_SG
void j_setup(char *, char *, char *, char *, char *);
#else
void j_setup(void);
#endif
	
void j_disconnect(void);


void cmd_send(char *arg);
void message_parse(ikspak *pak);


void cmd_presence(char *arg);
void cmd_subscribe(char *arg);
void cmd_subscribed(char *arg);
void cmd_unsubscribe(char *arg);
void cmd_remove(char *arg);
void cmd_name(char *arg);
void presence_parse(ikspak *pak);


void menu_command(char *buf);
void cmd_help(char *arg);

void cmd_option(char *arg);
void cmd_option_save(char *arg);
void cmd_option_load(char *arg);
void parse_options_from_server(ikspak *pak);

#ifdef USE_PERL
void cmd_perl(char *arg);
void pl_handle_message(char *from, char *body);
void pl_setup(void);
void pl_cleanup(void);
#else
#define pl_handle_message(x,y) do {} while(0);
#define pl_setup() do {} while(0);
#define pl_cleanup() do {} while(0);
#endif


void roster_fetch(void);
void roster_update(ikspak *pak);
void roster_update_presence(ikspak *pak);
juser *roster_find(iksid *id);
juser *roster_find_by_name(char *name, int isonline);
void cmd_roster(char *arg);
void cmd_group(char *arg);

void cmd_browse(char *arg);
void browse_parse(ikspak *p);

void cmd_senditem(char *arg);

void cmd_join(char *arg);
void cmd_part(char *arg);
void cmd_nick(char *arg);
void cmd_whisper(char *arg);
void gchat_send(textbuf *tb, char *msg);
void gchat_msg(ikspak *pak);
void gchat_whisper(textbuf *tb, ikspak *pak);
int gchat_pres(ikspak *pak);

textbuf *target_textbuf(ikspak *pak);

void cmd_search(char *arg);
void cmd_fields(char *arg);

void search_parse(ikspak *pak);
void search_error(ikspak *pak);

#define list_first(x) (void *)(*( x ))->first
#define list_next(x) (void *)( x )->node.next
#define list_prev(x) (void *)( x )->node.prev
listitem *list_append(list **lis, void *data);
listitem *list_prepend(list **lis, void *data);
void list_remove(list **lis, void *data);
void list_remove_all(list **lis);

void cmd_version(char *arg);
void send_version(iksid *to);
void version_parse(ikspak *pak);
void version_error(ikspak *pak);

void on_exec(char *command);

void cmd_chat(char *arg);
void cmd_close(char *arg);
int chat_msg(ikspak *pak);
void chat_send(textbuf *tb, char *msg);

void cmd_time(char *arg);
void send_time(iksid *to);
void time_parse(ikspak *pak);

#endif    /* JABBERX_COMMON_H */
