/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */

/* maximum number of rekey msgs for a rekeying */
#define MAX_REKEYMSGS	100

typedef struct RekeyMsgs_ {
    u8 store[MAX_REKEYMSGS][MAX_MSGSIZE];
    MsgRec msg[MAX_REKEYMSGS];
    int num;
    /* for user-/key-oriented rekeying */
    KNode *kNode[MAX_REKEYMSGS];
    int groupNum[MAX_REKEYMSGS];
} RekeyMsgs;

/****************************************************************************
 * function prototypes
 ****************************************************************************/

/* ks.c */
Server *newServer(int initDBSize);
int freeServer(Server *servRec);
Group *findGrp(Server *servRec, char *grpName);
int setupRekeyAddr(AddrRec *rekeyAddr);
int setupReqAddr(AddrRec *reqAddr);
int setupKGraph(Server *servRec);
int consMsgSig(MsgRec *msg, int msgNum, int numMsgs, u8 *authTree, MsgRec *blockSig);
int binaryTreeSigning(MsgRec *msgs, int numMsgs, SA *sigSA);
int encKNodeWithKNodes(KNode *kn, KNodeLst *knl, MsgRec *msg);
int rekeyKNode(KNode *kn, MsgRec *msg);
int rekeyKNodeDes(KNode *kn, MsgRec *msg);
int keyRekeyKNode(KNode *kn, MsgRec *msg, RekeyMsgs *rekeyMsgs);
int userRekey(Server *servRec, RekeyMsgs *rekeyMsgs);
int keyRekey(Server *servRec, RekeyMsgs *rekeyMsgs);
int graphRekeyMG(Server *servRec, RekeyMsgs *rekeyMsgs);
int groupRekeyMG(Server *servRec, RekeyMsgs *rekeyMsgs);
int sendRekeyMsg(Server *servRec);
int appendRekeySA2Anc(KNode *knode, u32 grpsRekey, MsgRec *msg);
int appendNewSALst(KNodeLst *knl, MsgRec *msg);
int appendDelSALst(KNodeLst *knl, MsgRec *msg);
int reqRekeyMsg(Server *servRec, Request *req, MsgRec *msg);
int sendReqUser(Server *servRec);
int sendResyn(Server *servRec, Request *req);
int updateKeyDes(Group *grp, KNode *knode, u32 *nextSPI);
int updateKey(Server *servRec);
int resetChangingDes(KNode *knode);
int resetRekeyedDes(KNode *knode);
int resetStatus(Server *servRec);
u8 *putGrpSA(Group *grp, u8 *curr);
int ackMsg(Server *servRec, MsgRec *msg, Request *req, int firstAck);
int authorizeReq(Server *servRec, Request *req);
int consReq(Server *servRec, Request *req, MsgRec *msg);
int selectTrees(Server *servRec, Request *req);
int procReq(Server *servRec, Request *req);
UserSkDB *newUserSkDB(int initSize);
int addUserSkDB(UserSkDB *userSkDB, int sk, User *user);
int deleteUserSkDB(UserSkDB *userSkDB, int sk);
