/*
    Keystone version 1.0 -- A group key management service
    Copyright (C) 2000  The University of Texas at Austin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.

    Keystone version 1.0 was written by Chung Kei Wong for a research project
    directed by Professor Simon S. Lam, Department of Computer Sciences.  
 */

/*
    For a description of the techniques implemented herein as well as a 
    performance analysis please see the article "Secure Group Communications
    Using Key Graphs," IEEE/ACM Transactions on Networking, 8(1):16-30,
    February 2000. (An early version in Proceedings ACM SIGCOMM '98.)
    Also see the article "Keystone: A Group Key Management Service,"
    in Proceedings of International Conference on Telecommunications
    ICT 2000, May 2000.

    Submit questions to:

	Dr. Chung Kei Wong                  Email:  ckwong@alumni.utexas.net
	Inktomi Corporation
	Foster City, CA 94080 USA

        or

        Professor Simon S. Lam              Email:  lam@cs.utexas.edu
        Dept. of Computer Sciences          Phone:  (512) 471-9531
        The University of Texas at Austin   Fax:    (512) 471-8885
        Austin, Texas 78712 USA

 */


#define MAX_BATCH_SIZE	16	/* max batch size */
/* max rekey message size = batch size * 1024 (heuristics) */
#define MAX_REKEY_SIZE	4096
#define MAX_UDP_SIZE	132	/* max data size in a UDP datagram */
#define RELI_HDR_SIZE	4	/* header size for reliable multicast */
#define MAX_RELI_DATA	128	/* MAX_UDP_SIZE - RELI_HDR_SIZE */

extern int nR1[21];
extern int nR2[21];

#define UDP_MCAST	1
#define TCP_UCAST	2

typedef struct {
    u16 msgNum;			/* rekey msg number */
    u8 numSrcUDPs, numUDPs;	/* number of src UDPs and received UDPs */
    u8 udpExist[MAX_REKEY_SIZE/MAX_RELI_DATA];
    u8 data[MAX_REKEY_SIZE];
} RekeyWorkUDP;

typedef struct {
    u8 data[MAX_REKEY_SIZE];
    int size;
} RekeyWorkTCP;

typedef struct {
    int maxRekey, numRekey;
    RekeyWorkUDP **rekeyWork;
    float alpha;
    int *nR, nRLen;
    struct AddrRec_ *rekeyAddr;
} UDPmcastCtx;

typedef struct {
    int maxRekey, numRekey;
    RekeyWorkTCP **rekeyWork;
    struct KNode_ *kNode;
    struct Group_ *grp;
    int sk;
} TCPucastCtx;

typedef struct RekeyCtx_ {
    int deliveryType;		/* delivery type */
    union {
	UDPmcastCtx *udpCtx;
	TCPucastCtx *tcpCtx;
    } ctx;
} RekeyCtx;


/* sndrekey.c */
int initRekeyCtxTCP(RekeyCtx *rekeyCtx, int maxRekey);
int initRekeyCtxUDP(RekeyCtx *rekeyCtx, int maxRekey, int nR[], int nRLen, float alpha);
RekeyCtx *newRekeyCtxTCP(int maxRekey);
RekeyCtx *newRekeyCtxUDP(int maxRekey, int nR[], int nRLen, float alpha);
int sendRekey(RekeyCtx *ctx, MsgRec *rekeyMsg);
int sendRekeyTCPusers(struct KNode_ *kn, MsgRec *rekeyMsg, int nonChanging);
int sendRekeyTCP(TCPucastCtx *ctx, MsgRec *rekeyMsg);
int sendRekeyUDP(UDPmcastCtx *ctx, MsgRec *rekeyMsg);
int recvRekey(RekeyCtx *rekeyCtx, MsgRec *rekeyMsg);
int recvRekeyTCP(TCPucastCtx *tcpCtx, MsgRec *rekeyMsg);
int rekeyFromUDP(MsgRec *rekeyMsg, UDPmcastCtx *udpCtx);
int recvRekeyUDP(UDPmcastCtx *udpCtx, MsgRec *rekeyMsg);
