/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** command list and menu
*/

#include "common.h"

struct menuitem
{
	char *shortcut;
	char *command;
	void (*func)(char *arg);
} menuitems[] =
{
	{ "h",  "help",       cmd_help },
	{ "q",  "quit",       cmd_quit },
	{ "co", "connect",    cmd_connect },
	{ "s",  "send",       cmd_send },
	{ "p",  "presence",   cmd_presence },
	{ "r",  "roster",     cmd_roster },
	{ "su", "subscribe",  cmd_subscribe },
	{ "sy", "subscribed", cmd_subscribed },
	{ "g",  "group",      cmd_group },
	{ "un", "unsubscribe",cmd_unsubscribe },
	{ "rem","remove",     cmd_remove },
	{ "n",  "name",	      cmd_name },
	{ "b",  "browse",     cmd_browse },
	{ "j",  "join",       cmd_join },
	{ "pr", "part",       cmd_part },
	{ "o",  "option",     cmd_option },
        { "po", "putoptions", cmd_option_save },
        { "go", "getoptions", cmd_option_load },
        { "t",  "time",       cmd_time },
        { "v",  "version",    cmd_version },

#ifdef USE_PERL
	{ "pl", "perl",       cmd_perl },
#endif

        { "ch", "chat",       cmd_chat },
        { "cl", "close",      cmd_close },
        { "sh", "search",     cmd_search },
        { "f",  "fields",     cmd_fields },
        { "si", "senditem",   cmd_senditem },
   
        { "w",  "whisper",    cmd_whisper },
 	{ "ni",  "nick",      cmd_nick },
 
	{ NULL, NULL, NULL}
};


static struct menuitem *menu_find(char *cmd)
{
	int i = 0;

	if(!cmd) return(NULL);

	while(menuitems[i].command)
	{
		if(strcasecmp(cmd, menuitems[i].shortcut) == 0 || strcasecmp(cmd, menuitems[i].command) == 0)
			return &menuitems[i];
		i++;
	}
	return NULL;
}


void cmd_oldhelp(char *arg)
{
	int i = 0, n;

	gui_attr(GA_MENU);

	if(arg)
	{
		struct menuitem *item;
		item = menu_find(arg);
		if(!item)
		{
			gui_print("Unknown command '%s'.\n", arg);
			return;
		}

        gui_print("Command help feature not implemented yet :-(\n");
        if (!options.compact_lines)
            gui_print ("\n");

		return;
	}

	gui_attr(GA_MENU);
	gui_print("Command list:\n");
	n = ui_w / 20;
	if(n == 0) n = 1;
	while(menuitems[i].shortcut)
	{
		gui_print("%-3s %-12s  ", menuitems[i].shortcut, menuitems[i].command);
		i++;
		if((i % n) == 0) gui_print("\n");
	}
	if((i % n) != 0) gui_print("\n");
	gui_print("Type 'help <cmdname>' for command specific help.\n");
      if (!options.compact_lines)
          gui_print ("\n");
}


void menu_command(char *buf)
{
	char *t,*p,*m;
	struct menuitem *item;

	t = strtok_r(buf," ",&p);

	item = menu_find(t);
	if(!item)
	{
		gui_attr(GA_NOTICE);
		gui_print("Unknown command.\n");
              if (!options.compact_lines)
                  gui_print ("\n");
		return;
	}

	t = strtok_r(NULL, "",&p);
	if(t)
	{
		m = strdup(t);
		item->func(m);
		free(m);
	}
	else
		item->func("");
}
