/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** jabber protocol
*/

#include "common.h"

struct netdata net;

#ifdef HAVE_SG
//keystoneSG keystoneclient;


int interactive(KSCtx *ksCtx)
{ /* begin of interactive() */
    char *grpName[MAX_GROUPS], *line, store[128], *command;
    int i, numGrps, request[MAX_GROUPS];

    /* fgets(line, 128, stdin); */
    while (1) {
        line = store;
        printf("Operation? ");
        fgets(line, 128, stdin);
        while (isspace(*line))
            line++;
        command = strtok(line, " \n");
        if (command == NULL) continue;

        /* join/leave request */
        if ((!strncmp(command, "join", strlen("join"))) ||
            (!strncmp(command, "leave", strlen("leave")))) {
            numGrps = 0;
            while (numGrps < MAX_GROUPS) {
                grpName[numGrps] = strtok(NULL, " \n");
                if (grpName[numGrps] == NULL)
                    break;
                if ((!strncmp(grpName[numGrps], "join", strlen("join"))) ||
                    (!strncmp(grpName[numGrps], "leave", strlen("leave")))) {
                    command = grpName[numGrps];
                    continue;
                }
                if (!strncmp(command, "join", strlen("join"))) {
                    request[numGrps] = JOIN_GRP;
                } else if (!strncmp(command, "leave", strlen("leave"))) {
                    request[numGrps] = LEAVE_GRP;
                }
                numGrps++;
            }
            reqGroups(ksCtx, numGrps, grpName, request);
        }
        else
        /* resynchronize request */
        if (!strncmp(command, "resyn", strlen("resyn"))) {
            numGrps = 0;
            while (numGrps < MAX_GROUPS) {
                grpName[numGrps] = strtok(NULL, " \n");
                if (grpName[numGrps] == NULL)
                    break;
                request[numGrps] = RESYN_GRP;
                numGrps++;
            }
            reqGroups(ksCtx, numGrps, grpName, request);
        }
        else
        /* print group keys */
        if (!strncmp(command, "print", strlen("print"))) {
            numGrps = 0;
            while (numGrps < MAX_GROUPS) {
                grpName[numGrps] = strtok(NULL, " \n");
                if (grpName[numGrps] == NULL)
                    break;
                numGrps++;
            }
            for (i = 0; i < numGrps; i++) {
                Key *key;
                int size;
                char buf[80];
                key = getGroupKey(ksCtx, grpName[i], 0);
                if (key == NULL)
                    continue;
                printf("Key %s %d(%d): ", grpName[i], key->keyID, 
key->version);
                size = mputKey(key, buf);
                dumpKey(buf, size);
            }
        }
        else
        /* update group keys periodically */
        if (!strncmp(command, "update", strlen("update"))) {
            Key *key;
            int m = 20, size, lastVer[10];
            float p = 1.0;
            struct timeval sleepTime;
            char buf[80];
            numGrps = 0;
            while (numGrps < MAX_GROUPS) {
                lastVer[numGrps] = 0;
                grpName[numGrps] = strtok(NULL, " \n");
                if (grpName[numGrps] == NULL)
                    break;
                numGrps++;
            }
            printf("Time period (in seconds) and number of periods? ");
            scanf("%f %d", &p, &m);
            while (m-- > 0) {
                for (i = 0; i < numGrps; i++) {
                    key = getGroupKey(ksCtx, grpName[i], 0);
                    if (key == NULL) continue;
                    if (key->version == lastVer[i])
                        continue;
                    lastVer[i] = key->version;
                    printf("Key %s %d(%d): ", grpName[i],
                            key->keyID, key->version);
                    size = mputKey(key, buf);
                    dumpKey(buf, size);
                }
                /* sleep p seconds */
                sleepTime.tv_sec = p;
                sleepTime.tv_usec = (p - sleepTime.tv_sec) * 1000000;
                select(0, 0, 0, 0, &sleepTime);
            }
        }
        else
        /* quit */
        if (!strncmp(command, "quit", strlen("quit"))) {
            return 0;
        }
        else
        /* error */
        {
            if (*line != 0)
                fprintf(stderr, "ERROR: unknow command\n");
        }
    }
    return 0;
} /* end of interactive() */


static void initializeClient(char *ctxFile, char *keyFile, char *certFile, char* caFile, char *caPath)
{
 keystoneclient.ksCtx = newKSCtx(ctxFile);
 sslInfoInit(&keystoneclient.sslInfo, keyFile, certFile, caFile, caPath);
 strcpy(keystoneclient.regAddr.addr, keystoneclient.ksCtx->reqAddr.addr);
 keystoneclient.regAddr.port = keystoneclient.ksCtx->reqAddr.port;
 registerSSL(keystoneclient.ksCtx, &keystoneclient.sslInfo, &keystoneclient.regAddr);
}
#endif

static void j_on_packet(void *udata, ikspak *pak);
static void j_on_log(void *udata, char *xml, int dir);
static void j_handle_iq(ikspak *pak);

#ifdef HAVE_SG
void j_setup(char *ctxFile, char *keyFile, char *certFile, char* caFile, char *caPath)
#else
void j_setup(void)
#endif
{
	memset(&net, 0, sizeof(struct netdata));
	
#ifdef HAVE_SG
	memset(&keystoneclient, 0, sizeof(struct KEYSTONE));
       (void) initializeClient(ctxFile, keyFile, certFile, caFile, caPath);
#if 0
	interactive(keystoneclient.ksCtx);
//#else
        keystoneclient.numGrps = 0;

        keystoneclient.grpName[keystoneclient.numGrps]= "g1";
        keystoneclient.request[keystoneclient.numGrps] = JOIN_GRP;
        if (keystoneclient.grpName[keystoneclient.numGrps] == NULL)
			exit(0);
      printf("num %d gprName %s req %d\n", keystoneclient.numGrps, keystoneclient.grpName[keystoneclient.numGrps], keystoneclient.request[keystoneclient.numGrps]);
       reqGroups(keystoneclient.ksCtx, keystoneclient.numGrps+1, keystoneclient.grpName[keystoneclient.numGrps], keystoneclient.request);
			exit(0);

#endif					

       
#endif

	net.parser = iks_jabber_new(&net, j_on_packet);
	if(!net.parser) exit(1);
	iks_set_logging(net.parser, j_on_log);

}


void cmd_connect(char *arg)
{
	iksid *id;

	gui_attr(GA_SYSMSG);

	if(arg)
		/* FIX ME: parse password if given */
		id = iks_id_new(NULL, arg);
		/* FIX ME: store arg for next time */
	else
		id = iks_id_new(NULL, prf.user);

	if(!id || !id->server || !id->user)
	{
		/* FIX ME: print an error */
		return;
	}
	if(!id->resource) id->resource = prf.resource;

	if(net.state != NET_OFF)
	{
		gui_print("Disconnected.\n");
		j_disconnect();
	}

	net.id = id;
	gui_print("Connecting to '%s'...\n", net.id->server);
	gui_refresh();
#ifdef HAVE_SSL
	if(!iks_connect_tcp_ssl(net.parser,net.id->server,0))
	{
		gui_print(stderr,"Could not open ssl!\n");
#endif
	if(!iks_connect_tcp(net.parser, net.id->server, 0))
	{
		j_disconnect();
		gui_print("Cannot connect!\n");
		return;
	}
#ifdef HAVE_SSL
	}
#endif 

#ifdef HAVE_SG

#endif
	
	net.state = NET_CONNECT;
	ui_status();
}


void j_disconnect(void)
{
	iks_disconnect(net.parser);
	net.state = NET_OFF;
	ui_status();
}


static void j_on_packet(void *udata, ikspak *pak)
{
	if(!pak)
	{
		gui_attr(GA_SYSMSG);
		gui_print("Disconnected!\n");
		j_disconnect();
		return;
	}

	switch(pak->type)
	{
	case IKS_PAK_STREAM:
	{
		iks *x;
		net.state = NET_STREAM;
		ui_status();
		x = iks_make_auth(net.id, prf.pass, iks_find_attrib(pak->x, "id"));
		iks_insert_attrib(x, "id", "auth");
		iks_send(net.parser, x);
		iks_delete(x);
		gui_attr(GA_SYSMSG);
		gui_print("Connected, logging in...\n");
		break;
	}

	case IKS_PAK_MESSAGE:
		message_parse(pak);
		break;

	case IKS_PAK_PRESENCE:
		roster_update_presence(pak);
		break;

	case IKS_PAK_S10N:
		presence_parse(pak);
		break;

	case IKS_PAK_IQ:
		j_handle_iq(pak);
		break;

	case IKS_PAK_ERROR:
		gui_attr(GA_SYSMSG);
		if( iks_has_children(pak->x) && iks_cdata(iks_child(pak->x)) )
			gui_print("Stream:error: %s\n",iks_cdata(iks_child(pak->x)));
		else
			gui_print("Stream:error without error-text.\n");
		if( !options.compact_lines )
			gui_print("\n");
		break; 

	case IKS_PAK_UNKNOWN:
		gui_attr(GA_SYSMSG);
		gui_print("Stream:Got Unknown PAK\n");
		break;

	}

	gui_refresh();
}


static void j_on_log(void *udata, char *xml, int dir)
{
	FILE *f;

	if(!prf.logfile) return;
	f = fopen(prf.logfile, "a");
	if(f)
	{
		if(dir == IKS_DIR_IN) fprintf(f, "[RECV]\n"); else fprintf(f, "[SEND]\n");
		fprintf(f, "%s\n\n", xml);
		fclose(f);
	}
}


static void j_handle_iq(ikspak *pak)
{
	char *id = iks_find_attrib(pak->x, "id");

	if(iks_strcmp(id, "register") == 0)
	{
		if(pak->subtype == IKS_TYPE_RESULT)
		{
			net.state = NET_AUTH; /* FIX ME */
			gui_attr(GA_SYSMSG);
			gui_print("User account registered.\n");
			if (!options.compact_lines)
				gui_print ("\n");
			/* FIX ME: send auth */
		}
		else
		{
			gui_attr(GA_SYSMSG);
			gui_print("Registration failed.\n");
			/* FIX ME: error code */
		}
    }

	else if(iks_strcmp(id, "auth") == 0)
	{
		if(pak->subtype == IKS_TYPE_RESULT)
		{
			net.state = NET_ON;
			ui_status();
			gui_attr(GA_SYSMSG);
			if( options.fetch_options )
			{
				gui_print("Authorized, fetching options and roster...\n");
				cmd_option_load(NULL);
			} 
			else
				gui_print("Authorized, fetching roster...\n");
			/* agents_fetch(); */
			roster_fetch();
		}
		else
		{
			iks *x;
			gui_attr(GA_SYSMSG);
			gui_print("Authorization failed.\n");
			x = iks_find(pak->x, "error");
			if(x)
			{
				gui_print("Error %s: %s.\n", iks_find_attrib(x, "code"), iks_cdata(iks_child(x)));
				if (!options.compact_lines)
					gui_print ("\n");

			}
			/* FIX ME: switch to NET_OFF state */
		}
	}

	else if(iks_strcmp(pak->ns, IKS_NS_VERSION) == 0)
	{
		if(pak->subtype == IKS_TYPE_GET)
		{
                        send_version(pak->from);
		}
                else if (pak->subtype == IKS_TYPE_RESULT)
                {
                        version_parse(pak);
                }
                else if (pak->subtype == IKS_TYPE_ERROR) 
                {
                        version_error(pak);
                }
	}

	else if(iks_strcmp(iks_find_attrib(iks_child(pak->x), "xmlns"), IKS_NS_BROWSE) == 0)
	{
		browse_parse(pak);
	}

	else if(iks_strcmp(pak->ns, IKS_NS_SEARCH) == 0)
	{
                if (pak->subtype == IKS_TYPE_RESULT)
                {
                        search_parse(pak);
                }
                else if (pak->subtype == IKS_TYPE_ERROR) 
                {
                        search_error(pak);
                }
	}

	else if(iks_strcmp(pak->ns, IKS_NS_ROSTER) == 0)
	{
		roster_update(pak);
	}

	else if(iks_strcmp(pak->ns, IKS_NS_TIME) == 0)
	{
                if(pak->subtype == IKS_TYPE_GET)
                        send_time(pak->from);
                else if (pak->subtype == IKS_TYPE_RESULT)
                        time_parse(pak);
	}

	else if(iks_strcmp(iks_find_attrib(iks_child(pak->x), "xmlns"), IKS_NS_PRIVATE) == 0)
	{
		parse_options_from_server(pak);
	}
}
