/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**                    2001 Bernhard R. Link <brl@pcpool00.mathematik.uni-freiburg.de>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** user interface
*/

#define _GNU_SOURCE 1

#include "common.h"
#include <unistd.h>

#define COLOURTAG '^'
#define TRUE 1
#define FALSE 0

static void append(textbuf *tb, char *data, int len);
static void renderlines(textbuf *tb, int line, int part, int ofs, int scrok);

list *textbufs;

textbuf *mainbuf = NULL;
textbuf *curbuf = NULL;


void tb_set_attr(textbuf *tb, enum gattrs attr)
{
	static unsigned char buf[2];
	bool insync;

	if( tb->curattr != attr )
	{
		insync = tb->insync;
		
		buf[0] = COLOURTAG;
		buf[1] = '0' + attr;
		append(tb, buf, 2);

		tb->insync = insync;
	}
	tb->curattr = attr;
}

void tb_getname(char *dest, size_t len, textbuf *tb)
{
        switch (curbuf->type) 
        {
        case TB_MAIN:
		snprintf(dest,len, "JabberX "VERSION);
                break;
	case TB_GCHAT:
		snprintf(dest,len, "GChat [%s@%s]", curbuf->id->user, curbuf->id->server);
                break;
        case TB_CHAT:
		snprintf(dest,len, "Chat [%s@%s]", curbuf->id->user, curbuf->id->server);
                break;
        }
}

textbuf *tb_new(void)
{
	textbuf *tb;

	tb = malloc(sizeof(textbuf));
	if(!tb) return(NULL);
	memset(tb, 0, sizeof(textbuf));
	tb->buf = malloc(8192);
	if(!tb->buf)
	{
		free(tb);
		return NULL;
	}
	tb->bufmax = 8192; // Hm, was this missing an error, or is it now?
	tb->lineinfos = calloc(100,sizeof(lineinfo));
	if(!tb->lineinfos)
	{
		free(tb->buf);
		free(tb);
		return NULL;
	}
	tb->limax = 100;
	list_append(&textbufs, tb);

	return tb;
}


void tb_show(textbuf *tb)
{
	curbuf = tb;
	tb_bottom(tb);
	ui_status();

}


void tb_delete(textbuf *tb)
{
	if(curbuf == tb)
	{
		curbuf = list_next(curbuf);
		if(!curbuf) curbuf = mainbuf;
		tb_render(curbuf);
		ui_status();
	}
	list_remove(&textbufs, tb);
        if (tb->thread)
                free(tb->thread);
        /* FIXME: free the ID. */
	free(tb->buf);
	free(tb->lineinfos);
	free(tb);
}


void tb_puts(textbuf *tb, char *str)
{
	int len;
	bool insync;
	int l,s,o;
	int v;

	len = strlen(str);
	if(len == 0) return;

	insync = tb->insync && tb==curbuf;
	if( insync )
	{
		/* remember where we currently are */
		l = curbuf->curline;
		s = curbuf->curpart;
		o = curbuf->lineinfos[curbuf->curline].visiblechars - s*ui_w;

//	fprintf(stderr,"tb_puts('%10.20s',%d): l=%d, s= %d o= %d \n",str,len,l,s,o);

		/* add the data */
		append(tb, str, len);

		tb->curline = tb->lines;
		if( (v = tb->lineinfos[tb->curline].visiblechars) )
			tb->curpart = (v-1)/ui_w;
		else 
			tb->curpart = 0;

		/* draw it till the current position (which is the end) */
		renderlines(tb,l,s,o,1);

//	fprintf(stderr,"/tb_puts l=%d,s=%d \n",curbuf->curline,curbuf->curpart);

		tb->insync = TRUE;

	} else
		append(tb, str, len);

}


void pos_adjust(textbuf *tb, int *line, int *part)
{
	int maxpart,maxrest;

	if( *line > tb->lines )
	{
		*line = tb->lines;
#ifdef DEBUG
		fprintf(stderr,"strange part!\n\n");
#endif
	}
	else if( *line < 0 )
	{
		*line = 0;
#ifdef DEBUG
		fprintf(stderr,"strange line!\n\n");
#endif
	}

	if( tb->lineinfos[*line].visiblechars )
	{
		maxpart = (tb->lineinfos[*line].visiblechars-1) / ui_w ;
		maxrest = (tb->lineinfos[*line].visiblechars-1) % ui_w ;
	} else {
		maxpart = 0;
		maxrest = 0;
	}

	if( *part > maxpart )
	{
		*part = maxpart;
#ifdef DEBUG
		fprintf(stderr,"strange part!\n\n");
#endif
	}
	else if( *part < 0 )
	{
		*part = 0;
#ifdef DEBUG
		fprintf(stderr,"strange part!\n\n");
#endif
	}

}

void pos_minuslines(textbuf *tb, int *line , int *part, int *todo)
{
	int h,v;

	if( *part >= *todo )
	{
		*part -= *todo;
		
	} else if( *line == 0 )
	{
		*line = -1;
		*todo -= *part+1;
		*part = 0;
	}else {
		/* First take the rest of the current line */
		*todo -= *part+1;
		(*line)--;

		/* Then go up full lines, as long as possible */
//	fprintf(stderr,"ml: %d:%d \n",*todo,*line);

		while( *todo > 0 && *line >= 0 && ( *todo > (h=((v=tb->lineinfos[*line].visiblechars))?(((v-1)/ui_w)+1):1 )) )
		{
//		fprintf(stderr,"ml: %d:%d -> %d:%d\n",*todo,*line,*todo-h,*line-1);
			*todo -= h;
			(*line)--;
		}
//	fprintf(stderr,"ml: %d:%d \n",*todo,*line);

		/* Then go up parts of the remaining line, if any */

		if( *line < 0 )
			*part = 0;
		else if( (v = tb->lineinfos[*line].visiblechars) )
			*part = (v-1)/ui_w +1 - *todo;
		else
			*part = 0;
//	fprintf(stderr,"ml: %d as part, %d todo\n",*part,*todo);

	}
}


/*  Send all data from line line, part part, character ofs till tb->curline,tb->curpart 
 *  to ui_win_render to be displayed.  */

static void renderlines(textbuf *tb, int line, int part, int ofs, int scrok)
{
	int v;

//fprintf(stderr,"renderlines(lines=%d, part=%d, ofs=%d, scrok=%d);\n",line,part,ofs,scrok);
	while( line < tb->curline || ((line==tb->curline)&&(part<tb->curpart)) )
	{
		ui_win_render(tb->lineinfos[line].startattr,
				&(tb->buf[tb->lineinfos[line].startofs]),
				ui_w,ofs,part*ui_w+ofs);
		ofs = 0;
		part++;
		if( (v=tb->lineinfos[line].visiblechars)==0 || part > (v-1)/ui_w )
		{
			part = 0;
			line++;
		}
		ui_win_next(scrok);
	}
	
	/* And now the last line (which is the last i.e. no ui_win_next to be called
	 * and may not be terminated, if it is the very last) */
	if( (v=tb->lineinfos[line].visiblechars) )
		ui_win_render(tb->lineinfos[line].startattr,
				&(tb->buf[tb->lineinfos[line].startofs]),
				((v-1)%ui_w)+1,ofs,part*ui_w+ofs);
	ui_win_refresh();

	tb->insync = FALSE;
//fprintf(stderr,"/renderlines;\n");
}

/*  Redisplay the curren screen: */

void tb_render(textbuf *tb)
{
	int i,l,s;

	ui_win_reset();


//fprintf(stderr,"tb_render with line: %d part: %d\n",tb->curline,tb->curpart);
	
	/* adjust the current location: */

	pos_adjust(tb, &tb->curline, &tb->curpart );

//fprintf(stderr,"tb_render adjusted line: %d part: %d\n",tb->curline,tb->curpart);
	/* find the beginning of the window (note that curline now means the last line in the window): */

	i = ui_h - 5;  // What remains on lines
	l = tb->curline; // the current last line
	s = tb->curpart;

	pos_minuslines(tb,&l,&s,&i);

//fprintf(stderr,"tb_render starts at line: %d part: %d rest %d\n",l,s,i);

	/*  Now print the lines, till we reach our position: */

	if( l < 0 )
	{
		while( i>0 )
		{
			ui_win_next(0);
			i--;
		}
		l = 0;
		s = 0;
	}
	renderlines(tb,l,s,0,0);

}


void tb_forward(textbuf *tb, int todo)
{
	int h,v;

	/* Are we are still in the same line afterwards: */
	
	h=((((v=tb->lineinfos[tb->curline].visiblechars))?(((v-1)/ui_w)+1):(1)));
	if( tb->curpart+todo < h )
	{
		tb->curpart += todo;
	} else if ( tb->curline+1>= tb->lines ) {
		tb_bottom(tb);
		return;
	} else {

		/* First take the rest of the current line */
		todo -=  h-tb->curpart;
		tb->curpart = 0;
		(tb->curline) ++;
		/* Then take fulle lines, as long as possible */
	
		while( todo > 0 && tb->curline < tb->lines && ( todo > (h=((v=tb->lineinfos[tb->curline].visiblechars))?(((v-1)/ui_w)+1):1 )) )
		{
			todo -= h;
			(tb->curline)++;
		}
		/* Then go down parts of the remaining line, if any */

		if( tb->curline >= tb->lines  ) {
			tb_bottom(tb);
		}
		else if( (v = tb->lineinfos[tb->curline].visiblechars) )
			tb->curpart = todo;
		else
			tb->curpart = 0;
		
	}
	tb_render(tb);
}


void tb_backward(textbuf *tb, int lines)
{
	pos_minuslines(tb,&(tb->curline),&(tb->curpart),&lines);
	if( tb->curline < 0 )
		tb->curline = 0;
	tb_render(tb);
}


void tb_top(textbuf *tb)
{
	tb->curpart = 0;
	tb->curline = 0;
	tb_render(tb);
}


void tb_bottom(textbuf *tb)
{
	tb->curline = tb->lines;
	if( tb->lineinfos[tb->curline].visiblechars>0 )
		tb->curpart = (tb->lineinfos[tb->curline].visiblechars-1)/ui_w;
	else 
		tb->curpart = 0;
	tb_render(tb);
	tb->insync = TRUE;
}


static void append(textbuf *tb, char *data, int len)
{
	char *p;
	int i,newlines,bytesinline;
	int washat;

	if(tb->bytes + len + 1 >= tb->bufmax)
	{
		char *tmp;
		tmp = malloc(tb->bufmax + 8192);
		if(!tmp) return;
		memcpy(tmp, tb->buf, tb->bytes);
		free(tb->buf);
		tb->buf = tmp;
		tb->bufmax += 8192;
	}

	memcpy(&tb->buf[tb->bytes], data, len);

	newlines = 0;
	bytesinline = 0;
	washat=0;
	p = data;
	for(i=0;i<len;i++)
	{
		if( (*p != COLOURTAG && !washat ) || (*p == COLOURTAG && washat))
			bytesinline++;
		washat = *p == COLOURTAG;
		if( *p == '\n' )
		{
			newlines++;
			if(tb->lines + newlines + 1 >= tb->limax)
			{
				lineinfo *tmp;
				tmp = calloc(tb->limax + 128,sizeof(lineinfo));
				if(!tmp)
					return;
				memcpy(tmp,tb->lineinfos,tb->limax*sizeof(lineinfo));
				free(tb->lineinfos);
				tb->lineinfos = tmp;
				tb->limax += 128;
			}
			tb->lineinfos[tb->lines+newlines-1].visiblechars += bytesinline;
			tb->lineinfos[tb->lines+newlines].startofs = tb->bytes+i+1;
			tb->lineinfos[tb->lines+newlines].startattr = tb->curattr;
			tb->lineinfos[tb->lines+newlines].visiblechars = 0;
			bytesinline = 0;
		}
		p++;
	}

	tb->lineinfos[tb->lines+newlines].visiblechars += bytesinline;
	tb->bytes += len;
	tb->lines += newlines;
	tb->buf[tb->bytes] = '\0';
}
