/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** user interface
*/

#define _GNU_SOURCE 1

#include "common.h"
#include <langinfo.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#ifndef USE_SLANG
#undef bool
#include <curses.h>
#else
#undef bool
#include <slcurses.h>
#endif

#define PROMPT_TEXT "jx> "
#define PROMPT_LEN 4
#define SWAP_OVERLAP 5
#define COLOURTAG '^'

static WINDOW *mwin;
static WINDOW *swin;
static WINDOW *cwin;
static iksconv *conv;

static void ui_key(int);
static int ui_setup(void);
static void ui_free(void);

static void color(enum gattrs attr);

int ui_w, ui_h;
static int running = 1;

// For easier reply and multiple msgs:   - brl
#define LENaddbuf 512
struct tab_buf {char *addbuf; struct tab_buf *next; ikspool *pool;};
static struct tab_buf *tabbuf = NULL;

void cmd_quit(char *arg)
{
	running = 0;
}


int ui_run(void)
{
	struct timeval timeout; // To circumvent broken solaris-boxes...

	if(!ui_setup()) return(1);

	gui_attr(GA_SYSMSG);

	if(getuid() == 0 || getgid() == 0)
		gui_print("Becareful, you are using JabberX with root permissions!\n");

	gui_refresh();
	cmd_connect(NULL);
	gui_prompt();

	while(running)
	{
		fd_set fds, fds2;
		int sock;
		sock = iks_fd(net.parser);
		FD_ZERO(&fds);
		FD_ZERO(&fds2);
		FD_SET(0, &fds);
		if(sock != -1)
		{
			FD_SET(sock, &fds);
			if(net.state == NET_CONNECT) FD_SET(sock, &fds2);
		}

		timeout.tv_sec = 5;
		timeout.tv_usec = 0;
		if(select(sock + 2, &fds, &fds2, NULL, &timeout) > 0)
		{
			if(FD_ISSET(0, &fds))
			{
				int c;
				c = wgetch(cwin);
				if(c != ERR) ui_key(c);
			}

			if(sock != -1 && (FD_ISSET(sock, &fds) || FD_ISSET(sock, &fds2)))
			{
				iks_recv(net.parser, 0);
			}
		}
	}

	if(net.state != NET_OFF) j_disconnect();
	ui_free();

	return 0;
}

void ui_set_tab_content(char *fmt, ...)
{
	va_list ap;
	char addbuf[LENaddbuf];
	struct tab_buf *p;
	ikspool *ip;

	va_start(ap, fmt);
	vsnprintf(addbuf, sizeof(addbuf)-1, fmt, ap);
	va_end(ap);
	addbuf[sizeof(addbuf)-1] = 0;
	p=tabbuf;
	if( !p )
	{
		ip = iks_pool_new(512);
		p = iks_pool_alloc(ip,sizeof(struct tab_buf));
		p->pool = ip;
		p->next = p;
		p->addbuf = iks_pool_strdup(ip,addbuf);
		tabbuf = p;
	}
	else
	{
		if( strcmp(addbuf,p->addbuf) == 0 )
			return;
		while(p->next!=tabbuf)
		{
			p = p->next;
			if( strcmp(addbuf,p->addbuf) == 0 )
				return;
		}
		ip = p->pool;
		p->next = iks_pool_alloc(ip,sizeof(struct tab_buf));
		p = p->next;
		p->pool = ip;
		p->next = tabbuf;
		tabbuf = p;
		p->addbuf=iks_pool_strdup(ip,addbuf);
	}
}

static void ui_key(int c)
{
	static int pos = 0;
	static int posofs = 0;
	static unsigned char buf[1024];
	static int tabused = 0;
	int i,lastchar;
	unsigned char *p;
	unsigned char *tmp;

	switch(c)
	{
	case KEY_F(1):
		curbuf = mainbuf;
		ui_buffer_show(curbuf);
		break;

	case KEY_F(2):
		curbuf = list_next(curbuf);
		if(!curbuf) curbuf = mainbuf;
		ui_buffer_show(curbuf);
		break;

	case KEY_ENTER:
	case '\n':
	case '\r':
		tabused = 0;
		buf[pos] = '\0';
		if(pos == 0) break;
		if( !conv ) 
			tmp=buf;
		else
		if( !(tmp = iks_conv_utf8(conv, buf, pos)) ) 
		{
			gui_attr(GA_SYSMSG);
			gui_print("Could not convert. Try /o codeset=...\n");
			break;
		}
		if(tmp[0] == '/')
			menu_command(tmp+1);
		else if(curbuf->type == TB_GCHAT)
			gchat_send(curbuf, tmp);
                else if(curbuf->type == TB_CHAT)
			chat_send(curbuf, tmp);
		else 
			menu_command(tmp);
		pos = 0;
		posofs = 0;
		gui_prompt();
		break;

	case '\t':
		if( tabbuf )
		{
			if( tabused )
				tabbuf = tabbuf->next;
			tabused = 1;
			strcpy(buf,tabbuf->addbuf);
			posofs = 0;
			pos = strlen(buf);
			gui_prompt();
			p = buf;
			while( *p )
#ifndef USE_SLANG
				wechochar(cwin,*p++);
#else
				waddch(cwin,*p++);
			wrefresh(cwin);
#endif
		}
		break;

	case 0x08:
	case 0x7F:
	case KEY_BACKSPACE:
		if(pos == 0) break;
		pos--;
		buf[pos] = '\0';
		if(pos-posofs < SWAP_OVERLAP && posofs > 0) 
		{
			if( posofs > ui_w-PROMPT_LEN-SWAP_OVERLAP )
				posofs -= ui_w-PROMPT_LEN-SWAP_OVERLAP;
			else
				posofs = 0;
			wmove(cwin, 0, PROMPT_LEN);
			if( pos-posofs < ui_w-PROMPT_LEN )
				lastchar = pos;
			else
				lastchar = posofs+ui_w-PROMPT_LEN;
			for(i=posofs;i<lastchar;i++)
#ifndef USE_SLANG
				wechochar(cwin,buf[i]);
#else
				waddch(cwin,buf[i]);
			wrefresh(cwin);
#endif
			
		}
		else
		{
			wmove(cwin, 0, pos-posofs + PROMPT_LEN);
		}
		wclrtoeol(cwin);
		break;

	case KEY_NPAGE:
		tb_forward(curbuf, ui_h - 6 - 2);
		break;

	case KEY_PPAGE:
		tb_backward(curbuf, ui_h - 6 - 2);
		break;

	case KEY_HOME:
		tb_top(curbuf);
		break;

	case KEY_END:
		tb_bottom(curbuf);
		break;

	case KEY_RIGHT:
	case KEY_LEFT:
	case KEY_DOWN:
	case KEY_UP:
		break;

	default:
		if(c > 0x100) break;
		buf[pos] = (char)c;
		pos++;
		if( pos > ui_w+posofs-PROMPT_LEN-2 )
		{
			posofs += ui_w-PROMPT_LEN-SWAP_OVERLAP;
			wmove(cwin, 0, PROMPT_LEN);
			wclrtoeol(cwin);
			if( pos-posofs < ui_w-PROMPT_LEN )
				lastchar = pos;
			else
				lastchar = posofs+ui_w-PROMPT_LEN;
			for(i=posofs;i<lastchar;i++)
#ifndef USE_SLANG
				wechochar(cwin,buf[i]);
#else
				waddch(cwin,buf[i]);
			wrefresh(cwin);
#endif
		}
		else
		{
#ifndef USE_SLANG
		wechochar(cwin, c);
#else
		waddch(cwin,c);
		wrefresh(cwin);
#endif
		}

	}

	gui_refresh();
}


void optakt_codeset(char *name,void *data,int gui)
{
	if(!gui)
		return;  //Set later
	if( conv )
		iks_conv_delete(conv);
	conv = NULL;
	if( options.codeset ) 
		conv = iks_conv_new(options.codeset);
	if(!conv)
		conv = iks_conv_new(nl_langinfo(CODESET));
	if(!conv && gui) {
		gui_attr(GA_SYSMSG);
		gui_print("Could not initialise conversion\n");
	}
}


static int ui_setup(void)
{
    initscr();
    if( !options.no_colors ) {
	    start_color();
	    if( !has_colors() ) 
		    options.no_colors = 1;
    }
    cbreak();
    noecho();
    nonl();

    getmaxyx(stdscr, ui_h, ui_w);

	if( ui_w < PROMPT_LEN+SWAP_OVERLAP )
	{
		fprintf(stderr,"Too Small!\n");
		return 0;
	}

    if( !options.no_colors )
    {
	    init_pair(1, COLOR_CYAN, COLOR_BLACK);
	    init_pair(2, COLOR_RED, COLOR_BLACK);
	    init_pair(3, COLOR_GREEN, COLOR_BLACK);
	    init_pair(4, COLOR_WHITE, COLOR_BLACK);
	    init_pair(5, COLOR_MAGENTA, COLOR_BLACK);

	    init_pair(6, COLOR_WHITE, COLOR_BLUE);
    }

	mwin = newwin(ui_h - 5, 0, 0, 0);
#ifndef USE_SLANG
	idlok(mwin, TRUE);
#endif
	scrollok(mwin, FALSE);

	swin = newwin(2, 0, ui_h - 5, 0);
	scrollok(swin, FALSE);
    if( !options.no_colors )
    {
#ifndef USE_SLANG
	wbkgd(swin, COLOR_PAIR(6));
#else
	wattrset(swin, COLOR_PAIR(6));
#endif
    }

	cwin = newwin(3, 0, ui_h - 3, 0);
#ifndef USE_SLANG
	idlok(cwin, TRUE);
#endif
	scrollok(cwin, TRUE);
	keypad(cwin, TRUE);

	nodelay(cwin, TRUE);

	mainbuf = tb_new();
	if(!mainbuf) return(0);
	curbuf = mainbuf;
	tb_show(mainbuf);

	conv = NULL;
	optakt_codeset("codeset",&options.codeset,1);

	return 1;
}


static void ui_free(void)
{
	if( conv )
		iks_conv_delete(conv);

	resetterm();
	echo();
	endwin();
}


void gui_prompt(void)
{
	wmove(cwin, 0, 0);
    if( !options.no_colors )
    {
    wattrset(cwin, COLOR_PAIR(1));
    }
    wprintw(cwin, PROMPT_TEXT);
    if( !options.no_colors )
    {
    wattrset(cwin, COLOR_PAIR(4));
    }
	wclrtoeol(cwin);
    gui_refresh();
}

void ui_xprint(int uiflags, textbuf *tb, enum gattrs attr, char *fmt, ...)
{
	static unsigned char buf[2048];
	va_list ap;
	char *tmp;
	int i;
	struct tm *curtime;
	time_t timeinfo;
	

	if ( uiflags & UIF_SETATTR )
	{
		tb_set_attr(tb,attr);
		if( tb != mainbuf && uiflags & UIF_COPYMAIN)
			tb_set_attr(mainbuf,attr);
		if( tb != curbuf && curbuf != mainbuf && uiflags & UIF_COPYMAIN)
			tb_set_attr(curbuf,attr);
		if( tb == curbuf || uiflags & UIF_COPYMAIN)
			color(attr);
	}

	if (uiflags & UIF_TIMESTAMP && options.timestamps) {
		timeinfo=time(NULL);
		curtime=localtime(&timeinfo);
		snprintf(buf, 2048, "[%02d:%02d] ", curtime->tm_hour, curtime->tm_min);
		ui_buffer_puts(tb, buf);
	}

	if( fmt )
	{

		va_start(ap, fmt);
		vsnprintf(buf, sizeof(buf), fmt, ap);
		va_end(ap);
		buf[sizeof(buf)-1]=0;

		if( uiflags & UIF_STRIPNL )
		{
			i = strlen(buf);
			while( i>0 && buf[i-1] == '\n' )
			{
				i--;
				buf[i] = 0;
			}

		}
		if( uiflags & UIF_NEWLINE )
			strncat(buf,"\n",sizeof(buf));
		if( uiflags & UIF_EXTRANL && !options.compact_lines )
			strncat(buf,"\n",sizeof(buf));

		if( conv && (tmp = iks_conv_locale(conv, buf, strlen(buf))))
		{
			tb_puts(tb, tmp);
			if( tb != mainbuf && uiflags & UIF_COPYMAIN)
				ui_buffer_puts(mainbuf, tmp);
			if( tb != curbuf && curbuf != mainbuf && uiflags & UIF_COPYMAIN)
				ui_buffer_puts(curbuf, tmp); 
		} else {
			ui_buffer_puts(tb,buf);
			if( tb != mainbuf && uiflags & UIF_COPYMAIN)
				ui_buffer_puts(mainbuf, buf);
			if( tb != curbuf && curbuf != mainbuf && uiflags & UIF_COPYMAIN)
				ui_buffer_puts(curbuf, buf); 
		}
	}
}


void gui_refresh(void)
{
    wnoutrefresh(mwin);
    wnoutrefresh(swin);
    wnoutrefresh(cwin);
    doupdate();
}


void ui_status(void)
{
	char *s1,*s2;
	char buf[500];


	switch(net.state)
	{
	case NET_OFF: s1 = "Offline"; break;
	case NET_CONNECT: s1 = "Connecting"; break;
	case NET_STREAM: s1 = "Streaming"; break;
	case NET_REG: s1 = "Registering"; break;
	case NET_AUTH: s1 = "Authorizing"; break;
	case NET_ON: s1 = "Online"; break;
	default: s1 = "Error";
	}

	if( (s2=iks_id_print(net.id)) )
		snprintf(buf, sizeof(buf), "%s [%s]", s1, s2);
	else
		snprintf(buf, sizeof(buf), "%s [<none>]",s1);

	werase(swin);
	mvwaddstr(swin, 1, 0, buf);

	tb_getname(buf,sizeof(buf),curbuf);
	mvwaddstr(swin, 0, 0, buf);

	gui_refresh();
}


void ui_win_render(enum gattrs attr, char *text, int max, int shift, int ofs)
{
	int len = 0;
	unsigned char c;

	while(1)
	{
		if( len>=max+ofs || !text ) 
			return;
		c = *text;
		if(c == '\0' || c == '\n')
			return;
		if( len + shift  == ui_w + ofs ) 
			return;
		if( len == ofs )
			color(attr);
		len++;
		if(c == COLOURTAG)
		{
			len--;
			text++;
			if( *text != COLOURTAG)
			{
				if( len <= ofs )
					attr = *text - '0';
				else
					color(*text - '0');
				text++;
			}
		}
		else
		{
			if( len > ofs) {
				waddch(mwin, c);
			}
			text++;
		}
	}
}

int wmline;

void ui_win_reset()
{
	werase(mwin);
	color(GA_BODY);
	wmove(mwin,0,0);
	wmline=0;
}
void ui_win_next(int scroll)
{
	
	if(scroll) {
	// wrefresh(mwin);
		scrollok(mwin, TRUE);
		wscrl(mwin, 1);
		scrollok(mwin, FALSE);
	// wrefresh(mwin);
	}
	if( !scroll)	
	wmline++;
	wmove(mwin,wmline,0);
	if( scroll)
	{
	// wrefresh(mwin);
	}
}
void ui_win_refresh()
{
	wrefresh(mwin);
}


static void color(enum gattrs attr)
{
    if( options.no_colors )
	   return;
    switch(attr)
	{
    case GA_BODY:
        wattrset(mwin, COLOR_PAIR(4));
        break;

    case GA_HEADER:
        wattrset(mwin, A_BOLD | COLOR_PAIR(1));
        break;

    case GA_SYSMSG:
        wattrset(mwin, A_BOLD | COLOR_PAIR(3));
        break;

    case GA_MENU:
        wattrset(mwin, COLOR_PAIR(5));
        break;

    case GA_NOTICE:
        wattrset(mwin, COLOR_PAIR(2));
        break;
    }
}
